package cn.com.qmth.stmms.ms.core.repository;

import cn.com.qmth.stmms.ms.core.domain.ChangeLevel;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public interface ChangeLevelRepo extends JpaRepository<ChangeLevel, Long>, JpaSpecificationExecutor {

    @Query("select s from ChangeLevel s where s.workId = ?1 and s.subject = ?2 and s.stage = ?3 and s.auditStatus = 1")
    List<ChangeLevel> findByWorkIdAndSubjectAndAuditStatusAndStage(Long workId, String subject, MarkStage stage);

    @Query("select s from ChangeLevel s where s.workId = ?1 and s.subject = ?2 and s.stage = ?3 and s.paperId = ?4 and s.auditStatus = 1 and s.isCurr = 1")
    ChangeLevel findByWorkIdAndSubjectAndPaperIdAndAuditStatusAndStageAndIsCurr(Long workId, String subject, MarkStage stage, Long paperId);

    @Query("select s from ChangeLevel s where s.workId = ?1 and s.subject = ?2 and s.stage = ?3 and s.role = ?4 and s.paperId = ?5 and s.auditStatus = 1 and s.isCurr = 1")
    ChangeLevel findByWorkIdAndSubjectAndPaperIdAndAuditStatusAndStageAndRoleAndIsCurr(Long workId, String subject, MarkStage stage, Role role, Long paperId);

    void deleteByWorkIdAndSubjectAndStage(Long workId, String subject, MarkStage stage);

    @Query("select s from ChangeLevel s where s.workId = ?1 and s.subject = ?2 and s.stage = ?3 and s.paperId = ?4 and s.paperCheckId = ?5 and s.auditStatus = 1 and s.isCurr = 1")
    ChangeLevel findByWorkIdAndSubjectAndStageAndPaperIdAndPaperCheckIdAndAuditStatusAndIsCurr(Long workId, String subject, MarkStage stage, Long paperId, Long paperCheckId);

    @Query("select s from ChangeLevel s where s.workId = ?1 and s.subject = ?2 and s.stage = ?3 and s.paperId = ?4")
    List<ChangeLevel> findAllByWorkIdAndSubjectAndStageAndPaperId(Long workId, String subject, MarkStage stage, Long paperId);

    @Query("select s from ChangeLevel s where s.workId = ?1 and s.subject = ?2 and s.stage = ?3 and s.paperId = ?4 and s.paperCheckId = ?5 and s.auditStatus = 0")
    ChangeLevel findByWorkIdAndSubjectAndStageAndPaperIdAndPaperCheckId(Long workId, String subject, MarkStage stage, Long paperId, Long paperCheckId);

    List<ChangeLevel> findByWorkIdAndSubjectAndAuditStatusAndStage(Long workId, String subject, int auditStatus, MarkStage stage);

    @Query("select s from ChangeLevel s where (s.paperId, s.id) in (select cl.paperId, max(cl.id) from ChangeLevel cl where cl.paperCheckId = ?1 group by cl.paperId) order by s.id desc")
    Page<ChangeLevel> findByPaperCheckId(Long paperCheckId, Pageable pageable);
}
