package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.commons.lock.LockService;
import cn.com.qmth.stmms.ms.commons.lock.LockType;
import cn.com.qmth.stmms.ms.core.domain.*;
import cn.com.qmth.stmms.ms.core.domain.user.Role;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Service
public class ChangeLevelTaskService {

    @Resource
    private ChangeLevelRepo changeLevelRepo;
    @Resource
    private ChangeLevelTaskRepo changeLevelTaskRepo;
    @Resource
    private MarkSubjectRepo markSubjectRepo;
    @Resource
    private PaperRepo paperRepo;
    @Resource
    private WorkRepo workRepo;
    @Resource
    private MarkLogRepo markLogRepo;
    @Resource
    private LockService lockService;


    @Transactional
    public Map confirmTask(Long changeLevelTaskId) {
        ChangeLevelTask changeLevelTask = changeLevelTaskRepo.findOne(changeLevelTaskId);
        if (changeLevelTask == null) {
            throw new RuntimeException("改档确认任务不存在");
        }
        MarkSubject markSubject = markSubjectRepo.findOne(changeLevelTask.getWorkId() + "-" + changeLevelTask.getSubject());
        if (!changeLevelTask.getStage().equals(markSubject.getStage())) {
            throw new RuntimeException("操作不成功，请退出重新登录后尝试");
        }

        Boolean status;
        try {
            lockService.waitlock(LockType.PAPER, changeLevelTask.getPaper().getId());
            status = this.confirm(changeLevelTask);
        } catch (Exception e) {
            throw new RuntimeException("改档确认任务异常");
        } finally {
            lockService.unlock(LockType.PAPER, changeLevelTask.getPaper().getId());
        }
        Map map = new HashMap<>();
        map.put("status", status);
        map.put("hasSelfTask", false);
        return map;
    }

    @Transactional
    public boolean confirm(ChangeLevelTask changeLevelTask) {
        changeLevelTask.setConfirmChangeStage(true);
        changeLevelTask.setConfirmOn(new Date());
        changeLevelTaskRepo.saveAndFlush(changeLevelTask);

        // 确认改档，只修改试卷最终档位，不改变评卷员已评档位值
        if (MarkStage.LEVEL.equals(changeLevelTask.getStage())) {
            if (changeLevelTaskRepo.countByPaperIdAndSubjectAndStageAndConfirmChangeStage(changeLevelTask.getPaper().getId(), changeLevelTask.getSubject(), changeLevelTask.getStage(), false) == 0) {
                paperRepo.updateRoughLevelById(changeLevelTask.getPaper().getId(), changeLevelTask.getResult());
            }
        } else if (MarkStage.SCORE.equals(changeLevelTask.getStage())) {
            if (changeLevelTaskRepo.countByPaperIdAndSubjectAndStageAndConfirmChangeStage(changeLevelTask.getPaper().getId(), changeLevelTask.getSubject(), changeLevelTask.getStage(), false) == 0) {
                paperRepo.updateLevelByIdAndShift(changeLevelTask.getPaper().getId(), changeLevelTask.getResult());
            }
        }

        // 记录日志
        Work work = workRepo.findOne(changeLevelTask.getWorkId());
        Paper paper = changeLevelTask.getPaper();
        ChangeLevel changeLevel = changeLevelRepo.getOne(changeLevelTask.getChangeLevelId());
        MarkLog markLog = new MarkLog(changeLevelTask.getWorkId(), work.getName(), paper.getId(), Subject.valueOf(changeLevelTask.getSubject()), changeLevelTask.getStage(), paper.getTest(), paper.getExamNumber(), paper.getStudentName(), changeLevelTask.getMarkerId(), changeLevelTask.getMarkerName(), Role.MARKER, MarkLogOperType.CHANGE_LEVEL.getId(), MarkLogOperType.CHANGE_LEVEL.getName(), changeLevel.getOriginalLevel(), changeLevelTask.getResult(), "", new Date());
        markLogRepo.save(markLog);
        return true;
    }

    public Object updateMarkById(Long markTaskId, Boolean isMark) {
        changeLevelTaskRepo.updateMarkTaskMarkById(markTaskId, isMark);
        return true;
    }
}
