package cn.com.qmth.stmms.ms.marking.service;

import cn.com.qmth.stmms.ms.core.cache.ParamCache;
import cn.com.qmth.stmms.ms.core.domain.Level;
import cn.com.qmth.stmms.ms.core.domain.MarkStage;
import cn.com.qmth.stmms.ms.core.domain.MarkSubject;
import cn.com.qmth.stmms.ms.core.domain.enums.TrialEnum;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskMarkerPaper;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskRoughLevel;
import cn.com.qmth.stmms.ms.core.domain.task.MarkTaskSelfCheck;
import cn.com.qmth.stmms.ms.core.repository.*;
import cn.com.qmth.stmms.ms.core.vo.Subject;
import cn.com.qmth.stmms.ms.marking.assembler.MarkTaskLevelAssembler;
import cn.com.qmth.stmms.ms.marking.assembler.MarkTaskRoughLevelAssembler;
import cn.com.qmth.stmms.ms.marking.dto.MarkTaskDTO;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Optional;
import java.util.Random;

/**
 * 自查卷服务
 */
@Service
public class MarkTaskSelfCheckService {

    @Resource
    private MarkTaskRoughLevelRepo markTaskRoughLevelRepo;

    @Resource
    private MarkTaskLevelRepo markTaskLevelRepo;

    @Resource
    private MarkTaskRoughLevelAssembler markTaskRoughLevelAssembler;

    @Resource
    private MarkTaskLevelAssembler markTaskLevelAssembler;

    @Resource
    private MarkTaskSelfCheckRepo markTaskSelfCheckRepo;

    @Resource
    private MarkTaskMarkerPaperRepo markTaskMarkerPaperRepo;

    @Resource
    private MarkerGroupLeaderRepo markerGroupLeaderRepo;

    @Resource
    private MarkSubjectRepo markSubjectRepo;

    @Resource
    private LevelRepo levelRepo;

    /**
     * @param markTaskDTO   任务对象
     * @param stage         阶段
     * @param taskId        任务ID
     * @param canCreateSelf 是否可以生成自查卷（批量操作使用）
     * @return 是否有自查卷
     */
    @Transactional
    public boolean updateMarkingCount(MarkTaskDTO markTaskDTO, MarkStage stage, Long taskId, boolean canCreateSelf) {
        Long workId = markTaskDTO.getWorkId();
        Subject subject = markTaskDTO.getSubject();
        Long markerId = markTaskDTO.getMarkerId();

        // 回评卷，不参与自查逻辑
        if (markTaskDTO.getReLevel()) {
            return false;
        }
        MarkSubject markSubject = markSubjectRepo.findOne(workId + "-" + subject.name());
        // 试评阶段，不参与自查逻辑
        if (markSubject.getTest() != TrialEnum.DEFAULT.getId()) {
            return false;
        }

        int selfCheckFrequency = 0;
        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
            selfCheckFrequency = ParamCache.roughLevelConfigMap.get(String.valueOf(workId)).getSelfCheckFrequency();
        } else if (MarkStage.LEVEL.equals(stage)) {
            selfCheckFrequency = ParamCache.levelConfigMap.get(String.valueOf(workId)).getSelfCheckFrequency();
        }

        // 自查卷频率为0时，默认不开启自查功能
        if (selfCheckFrequency == 0) {
            return false;
        }
        // 保存评卷任务
        addMarkerPaper(workId, subject, stage, markerId, taskId);

        int count = markTaskMarkerPaperRepo.countByWorkIdAndSubjectAndStageAndMarkerId(workId, subject, stage, markerId);
        // 更新分组中评卷老师评卷数量
        updateMarkerGroupLeaderCount(workId, subject, stage, markerId, count);

        boolean isReturnSelf = false;
        // 评卷数量等于频率值时
        if (canCreateSelf && count >= selfCheckFrequency) {
            // 生成自查卷评卷任务（当时批次中随机抽取一张）
            isReturnSelf = saveSelfCheckTask(workId, subject, stage, markerId);
        }
        return isReturnSelf;

    }

    public boolean saveSelfCheckTask(Long workId, Subject subject, MarkStage stage, Long markerId) {
        // 有未评的自查卷，直接返回
        MarkTaskSelfCheck markTaskSelfCheckHis = markTaskSelfCheckRepo.findByWorkIdAndMarkerIdAndSubjectAndStageAndMarking(workId, markerId, subject, stage, false);
        if(markTaskSelfCheckHis != null){
            return true;
        }

        List<MarkTaskMarkerPaper> markTaskMarkerPaperList = markTaskMarkerPaperRepo.findByWorkIdAndSubjectAndStageAndMarkerId(workId, subject, stage, markerId);
        Random r = new Random();
        // 随机抽一个任务
        int i = r.nextInt(markTaskMarkerPaperList.size());
        MarkTaskMarkerPaper markTaskMarkerPaper = markTaskMarkerPaperList.get(i);
        MarkTaskSelfCheck markTaskSelfCheck = new MarkTaskSelfCheck(workId, subject, markTaskMarkerPaper.getTaskId(), markerId, stage);
        if (markTaskSelfCheck != null) {
            markTaskSelfCheckRepo.save(markTaskSelfCheck);
        }
        return true;
    }

    public MarkTaskDTO getSelfCheckTask(Long workId, Long markerId, Subject subject, MarkStage stage) {
        MarkTaskSelfCheck markTaskSelfCheck = markTaskSelfCheckRepo.findByWorkIdAndMarkerIdAndSubjectAndStageAndMarking(workId, markerId, subject, stage, false);
        if (markTaskSelfCheck == null) {
            return null;
        }
        MarkTaskDTO markTaskDTO = null;
        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
            // 返回信息用正评任务中的信息
            MarkTaskRoughLevel markTaskRoughLevel = markTaskRoughLevelRepo.findOne(markTaskSelfCheck.getTaskId());
            markTaskDTO = markTaskRoughLevelAssembler.toSelfCheckDTO(markTaskRoughLevel);
            // id用自查表中id
            markTaskDTO.setId(markTaskSelfCheck.getId());
        } else if (MarkStage.LEVEL.equals(stage)) {
            MarkTaskLevel markTaskLevel = markTaskLevelRepo.findOne(markTaskSelfCheck.getTaskId());
            markTaskDTO = markTaskLevelAssembler.toSelfCheckDTO(markTaskLevel);
            // id用自查表中id
            markTaskDTO.setId(markTaskSelfCheck.getId());
        }
        return markTaskDTO;
    }

    @Transactional
    public boolean markingSelfCheckTask(Long id, String level) {
        if (StringUtils.isBlank(level)) {
            throw new RuntimeException("请选择档位");
        }
        MarkTaskSelfCheck markTaskSelfCheck = markTaskSelfCheckRepo.findOne(id);
        Long workId = markTaskSelfCheck.getWorkId();
        Subject subject = markTaskSelfCheck.getSubject();
        MarkStage stage = markTaskSelfCheck.getStage();
        Long markerId = markTaskSelfCheck.getMarkerId();
        List<Level> levels = levelRepo.findByWorkIdOrderByCode(workId);
        if (CollectionUtils.isEmpty(levels)) {
            throw new RuntimeException("未找到分档档位信息");
        }
        int levelValue;
        if (MarkStage.ROUGH_LEVEL.equals(stage)) {
            levelValue = Integer.parseInt(level);
        } else if (MarkStage.LEVEL.equals(stage)) {
            Optional<Level> optional = levels.stream().filter(m -> m.getCode().equals(level)).findFirst();
            if (!optional.isPresent()) {
                throw new RuntimeException("未找到档位[" + level + "]的档位值数据");
            }
            levelValue = optional.get().getLevelValue();
        } else {
            throw new RuntimeException("不支持的阶段");
        }
        markTaskSelfCheckRepo.updateMarkingById(id, level, levelValue, StringUtils.isNotBlank(level), new Date());

        // 更新分组中评卷老师评卷数量(清0)
        updateMarkerGroupLeaderCount(workId, subject, stage, markerId, 0);
        // 清空评卷明细数据（mark_task_marker_paper表）
        deleteMarkerPaper(workId, subject, stage, markerId);
        return true;
    }

    public boolean addMarkerPaper(Long workId, Subject subject, MarkStage stage, Long markerId, Long taskId) {
        List<MarkTaskMarkerPaper> markTaskMarkerPaperList = markTaskMarkerPaperRepo.findByWorkIdAndSubjectAndStageAndMarkerIdAndTaskId(workId, subject, stage, markerId, taskId);
        if (CollectionUtils.isEmpty(markTaskMarkerPaperList)) {
            MarkTaskMarkerPaper markTaskMarkerPaper = new MarkTaskMarkerPaper(workId, subject, stage, markerId, taskId);
            markTaskMarkerPaperRepo.save(markTaskMarkerPaper);
        }
        return true;
    }

    public boolean updateMarkerGroupLeaderCount(Long workId, Subject subject, MarkStage stage, Long markerId, int markingCount) {
        markerGroupLeaderRepo.updateMarkingCountByWorkIdAndSubjectAndStageAndMarkerId(workId, subject, stage, markerId, markingCount);
        return true;
    }

    public boolean deleteMarkerPaper(Long workId, Subject subject, MarkStage stage, Long markerId) {
        markTaskMarkerPaperRepo.deleteByWorkIdAndSubjectAndStageAndMarkerId(workId, subject, stage, markerId);
        return true;
    }

    @Transactional
    public void deleteSelfData(Long workId, Subject subject, MarkStage stage) {
        // 删除自查卷
        markTaskSelfCheckRepo.deleteByWorkIdAndSubjectAndStage(workId, subject, stage);
        // 删除评卷员当前评卷记录
        markTaskMarkerPaperRepo.deleteByWorkIdAndSubjectAndStage(workId, subject, stage);
    }
}
